@@ -1,17 +1,20 @@
 package SPOPS::DBI::RandomCode;
 
-# $Id: RandomCode.pm,v 1.15 2001/01/31 02:30:44 cwinters Exp $
+# $Id: RandomCode.pm,v 1.2 2001/02/20 04:39:16 lachoy Exp $
 
 use strict;
 use SPOPS  qw( _w );
 
 @SPOPS::DBI::RandomCode::ISA     = ();
-$SPOPS::DBI::RandomCode::VERSION = sprintf("%d.%02d", q$Revision: 1.15 $ =~ /(\d+)\.(\d+)/);
+$SPOPS::DBI::RandomCode::VERSION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
 
 my $DEFAULT_WIDTH = 8;
 
 sub pre_fetch_id  { 
   my ( $class, $width ) = @_;
+  warn "****SPOPS::DBI::RandomCode is deprecated and will probably be\n",
+       "removed in a future version of SPOPS. Please change your\n",
+       "classes to use 'SPOPS::Key::Random' instead.\n";
   $width ||= $class->CONFIG->{id_width} || $DEFAULT_WIDTH;
   my $code =  $class->generate_random_code( $width );
   _w( 1, "Created insert ID ($code)" );
